/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.adapter;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.HashMap;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.petals.admin.EsbEditor;
import org.objectweb.petals.admin.models.AgentModel;
import org.objectweb.petals.admin.models.ComponentModel;
import org.objectweb.petals.admin.tools.MsgBox;


/**
 * @author Maxime Carpentier
 *  Main controller - listen to GEF's models change and controls the ESBModel 
 * 
 */

public class EsbAdapter implements PropertyChangeListener {
	private EsbEditor esbeditor;
	
	//MBeanServerConnections
	private HashMap MBeanServerConnections = new HashMap();


	/**
	 * @return Returns the esbeditor.
	 */
	public EsbEditor getEsbeditor() {
		return esbeditor;
	}
	
	public void AddMBeanServerConnections(MBeanServerConnection mbsc,String desc) {
		MBeanServerConnections.put(desc,mbsc);
	}
	/**
	 * @param esbeditor The esbeditor to set.
	 */
	public void setEsbeditor(EsbEditor esbeditor) {
		this.esbeditor = esbeditor;
	}
	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * Invoke methods on components/agents on notification
	 */
	public void propertyChange(PropertyChangeEvent evt) {
	 	String method,target,container = "";
		 if (evt.getPropertyName().equals(AgentModel.P_INVOKE)) {
		 	method = (String) evt.getNewValue();
		 	target =  ((AgentModel) evt.getOldValue()).getName();
		 	container = ((AgentModel) evt.getOldValue()).getContainer();
		 	//System.out.println("Invoke() called (arg:" + method + ") - agent:" + target +" -container : " + container);
		 	MBeanServerConnection mbsc = (MBeanServerConnection) MBeanServerConnections.get(container);

			try {
			 	ObjectName on = new ObjectName("org.objectweb.petals:name="+target+",type=system");
				mbsc.invoke(on,method,null,null);
			} catch (MalformedObjectNameException e) {
				e.printStackTrace();
			} catch (NullPointerException e) {
				e.printStackTrace();
			} catch (InstanceNotFoundException e) {
				e.printStackTrace();
			} catch (MBeanException e) {
				MsgBox msg = new MsgBox(esbeditor.getSite().getWorkbenchWindow());
				msg.display("An error occured during the action","error");
				e.getTargetException().printStackTrace();
			} catch (ReflectionException e) {
				MsgBox msg = new MsgBox(esbeditor.getSite().getWorkbenchWindow());
				msg.display("An error occured during the action","error");
				e.getTargetException().printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} 
		 }
		 else if (evt.getPropertyName().equals(ComponentModel.P_INVOKE)) {
		 	method = (String) evt.getNewValue();
		 	target =  ((ComponentModel) evt.getOldValue()).getName();
		 	container = ((ComponentModel) evt.getOldValue()).getContainer();
		 	//System.out.println("Invoke() called (arg:" + method + ") - agent:" + target +" -container : " + container);
		 	MBeanServerConnection mbsc = (MBeanServerConnection) MBeanServerConnections.get(container);
			try {
			 	ObjectName on = new ObjectName("org.objectweb.petals:name="+target+",type="+((ComponentModel) evt.getOldValue()).getType());
				mbsc.invoke(on,method,null,null);
			} catch (MalformedObjectNameException e) {
				e.printStackTrace();
			} catch (NullPointerException e) {
				e.printStackTrace();
			} catch (InstanceNotFoundException e) {
				e.printStackTrace();
			} catch (MBeanException e) {
				MsgBox msg = new MsgBox(esbeditor.getSite().getWorkbenchWindow());
				msg.display("An error occured during the action","error");
				e.getTargetException().printStackTrace();
			} catch (ReflectionException e) {
				MsgBox msg = new MsgBox(esbeditor.getSite().getWorkbenchWindow());
				msg.display("An error occured during the action","error");
				e.getTargetException().printStackTrace();
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		 }
		
	}
}
