/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.gef.EditPolicy;
import org.objectweb.petals.admin.editpolicies.MyComponentEditPolicy;
import org.objectweb.petals.admin.editpolicies.MySelectionEditPolicy;
import org.objectweb.petals.admin.models.ComponentModel;
import org.objectweb.petals.admin.tools.ComponentFigure;



/**
 * @author Maxime Carpentier
 * Controller of a component model
 *
 */
public class ComponentEditPart extends EditPartWithListener {


	private Label label;

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * 	 initilaize the graphical representation of the model
	 */
	protected IFigure createFigure() {
		ComponentModel model = (ComponentModel) getModel();
		ComponentFigure figure = new ComponentFigure(model.getState());
		return figure;
	}
	
	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(ComponentModel.P_CONSTRAINT) || evt.getPropertyName().equals(ComponentModel.P_NAME))
			refreshVisuals(); 
		if (evt.getPropertyName().equals(ComponentModel.P_STATE)) {
			refreshFigure(); 
			refreshVisuals();	
			refreshChildren();
		}


	}
	public void refreshFigure() {
		((ComponentFigure) this.getFigure()).setImg(((ComponentModel) getModel()).getState().getImage());
	}
	
	public int getEditPartType() {
		return _COMPONENT_EDIT_PART;
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		    installEditPolicy(EditPolicy.COMPONENT_ROLE, new MyComponentEditPolicy());
		    installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, new MySelectionEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 * update the text of the label up to model's name property
	 */
	protected void refreshVisuals() {
		((ComponentFigure) this.figure).setName(((ComponentModel) getModel()).getName());
	}
	


}
