/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.objectweb.petals.admin.models.EsbModel;
import org.objectweb.petals.admin.models.ServiceContainerModel;
import org.objectweb.petals.admin.tools.CustomChopboxAnchor;
import org.objectweb.petals.admin.tools.EsbFigure;


/**
 * @author Maxime Carpentier
 */
public class EsbEditPart extends EditPartWithListener implements NodeEditPart {

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	public int getEditPartType() {
		return _ESB_EDIT_PART;
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * initilaize the graphical representation of the model
	 */
	protected IFigure createFigure() {
		EsbModel model = (EsbModel) getModel();
		EsbFigure figure = new EsbFigure(Math.max(model.getBottomSize(),model.getTopSize()));
		figure.setOpaque(true);
		((EsbModel) getModel()).setConstraint(new Rectangle(new Point(20,0),figure.getPreferredSize()));
		return figure;
	}

	  /**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * create a new anchor to the esb,regarding the connectionEditPart's source position
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
	  	if (connection.getSource() != null) {
	  		CustomChopboxAnchor ca = new CustomChopboxAnchor(getFigure(),((ServiceContainerModel) (connection.getSource()).getModel()).getConstraint());
	  		return ca;
	  		
	  	}
	  	return null;
	  }

	  /**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
	  	ChopboxAnchor ca = new ChopboxAnchor(getFigure());
	    return ca;
	  }

	  /**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
	    return ((EsbModel)getModel()).getModelTargetConnections();
	  }
	  
	  
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 * updates the figure's position relative to the model
	 */
	protected void refreshVisuals() {
		Rectangle constraint = ((EsbModel) getModel()).getConstraint();
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			constraint);
	}

	/**
	 * Update the size of the Esb figure relative to the containers
	 */
	protected void refreshFigure() {
		((EsbFigure) this.getFigure()).rebuild(Math.max(((EsbModel) getModel()).getBottomSize(),((EsbModel) getModel()).getTopSize()));
		((EsbModel) getModel()).setConstraint(new Rectangle(new Point(20,0),figure.getPreferredSize()));
}
	
	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(EsbModel.P_CONSTRAINT))
			refreshVisuals(); 
	    else if (evt.getPropertyName().equals(EsbModel.P_TARGET_CONNECTION))
	        refreshTargetConnections(); 
	    else if (evt.getPropertyName().equals(EsbModel.P_SIZE)) {
	    	refreshFigure(); 
	    }
}
	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return null;
	}
	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return null;
	}

}
