/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.petals.admin.adapter.EsbAdapter;
import org.objectweb.petals.admin.states.State;


/**
* @author Maxime Carpentier
*/
public class ComponentModel extends AbstractModel {
	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_FOCUS = "_focus";
	public static final String P_STATE = "_componentState";
	public static final String P_INVOKE = "_ComponentInvoke";
	public static final String P_NOM = "_component";
	public static final String P_TYPE = "_type";
	private final String P_ID = "_componentModell";
	private String name ="_myComponent";
	private String type = "_myType";
	private EsbAdapter esbcontroller;
	
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
	private Rectangle constraint;
	public static final String P_NAME = "_name";
	private String container;
	/**
	 * @return Returns the container.
	 */
	public String getContainer() {
		return container;
	}
	
	public void setState(State state) {
		this.state = state;
		firePropertyChange(P_STATE, this, state);
	}
	
	/**
	 * @param container The container to set.
	 */
	public void setContainer(String container) {
		this.container = container;
	}
	/**
	 * @see org.objectweb.petals.admin.models.AbstractModel#getState()
	 * @return the current state
	 */
	public State getState() {
		return state;
	}
	
	public void invoke(String method,String container) {
		this.container = container;
		firePropertyChange(P_INVOKE, this, method);
	}
	

	public ComponentModel(EsbAdapter esbcontroller) {
		this.addPropertyChangeListener(esbcontroller);
		//esbcontroller.
		this.esbcontroller = esbcontroller;
		//default state
		State state1 = esbcontroller.getEsbeditor().getState("esb.state.componentStopped");
		this.state = state1;

	}
	/*public ComponentModel(Address address,EsbAdapter esbcontroller) {
		this.address = address;
		this.addPropertyChangeListener(esbcontroller);
		//esbcontroller.
		State state1 = new State("component default state",new Image(null, EsbEditor.class.getResourceAsStream("img/componentStop.bmp")),esbcontroller.getEsbeditor());
		
		EsbAction action1 = new EsbAction((IWorkbenchPart) esbcontroller.getEsbeditor(),"run component method","img/run.bmp","exec","run");
		state1.addAction(action1);
		this.state = state1;

	}*/
	
	  /**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 * called to provide properties to the properties view
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
	  	firePropertyChange(P_FOCUS, null, null);
	    IPropertyDescriptor[] descriptors =
	      new IPropertyDescriptor[] {
	    		 new TextPropertyDescriptor(P_NAME, "name"), new TextPropertyDescriptor(P_TYPE, "type"), new TextPropertyDescriptor(P_STATE, "status")};
	    return descriptors;

	  }
		public Object getPropertyValue(Object id) {
			if (id.equals(P_NAME)) {
				return name;
			}
			else if (id.equals(P_TYPE)) {
				return type;
			}
			else if (id.equals(P_STATE)) {
				return getState().getId().toLowerCase();
			}
			return null;
		}

		public boolean isPropertySet(Object id) {
			if (id.equals(P_NAME))
				return true;
			else if (id.equals(P_TYPE))
				return true;
			else if (id.equals(P_STATE))
				return true;
			else
				return false;
		}

		public void setPropertyValue(Object id, Object value) {
			if (id.equals(P_NAME)) {
				setName((String) value);
			}
			if (id.equals(P_TYPE)) {
				setType((String) value);
			}
		}
		
	public ComponentModel(EsbAdapter esbcontroller,String caption) {
		this.addPropertyChangeListener(esbcontroller);
		this.esbcontroller = esbcontroller;
		//default state
		State state1 = esbcontroller.getEsbeditor().getState("esb.state.componentStopped");
		this.state = state1;
		this.setName(caption);
	}

	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	/**
	 * @return Returns the p_NAME.
	 */
	public static String getP_NAME() {
		return P_NAME;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
		firePropertyChange(P_NAME, null, name);
	}
	/**
	 * @param string
	 */
	public void setState(String string) {
		State state1 = esbcontroller.getEsbeditor().getState("esb.state.component"+string);
		setState(state1);
	}
}
