/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ScrollPane;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchWindow;
import org.objectweb.petals.admin.tools.ComponentFigure;


/**
 * @author Maxime Carpentier
 * This model is made to contain components
 *
 */
public class ComponentsModel extends AbstractModel {
	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_CHILDREN = "_children";
	public static final String P_FOCUS = "_focus";
	public static final String P_NAME = "_name";
	public final String P_ID = "_componentsModel";
	public static final int BORDER_SPACING=0; //bords du container
	public static final int SPACING=6; // espacement inter services
	public static int P_INIT_ROWS=1;
	public static int P_ROWS=1;
	public static int P_COLUMNS=3;
	private Rectangle constraint;
	private IWorkbenchWindow window;
	private List children = new ArrayList(); 
	private ServiceContainerDirectoryModel serviceDirectoryModel;
	
	/**
	 * @see org.objectweb.petals.admin.models.AbstractModel#getId()
	 */
	public String getId() {
		return P_ID;
	}
	
	/**
	 * @param child
	 */
	public void addChild(Object child) {
		children.add(child); 
		firePropertyChange(P_CHILDREN, null, null);
	}
	/**
	 * @param arg
	 * @param child
	 */
	public void moveChild(int arg,Object child) {
		children.remove(child);
		children.add(arg,child);
		firePropertyChange(P_CHILDREN, null, null);
	}
	/**
	 * @param sdm
	 */
	public ComponentsModel(ServiceContainerDirectoryModel sdm) {
		 serviceDirectoryModel = sdm;
	//	setConstraint(new Rectangle(new Point(0,0),CalculateInitialContainerSize()));
	}
	
	/**
	 * @param sdm
	 * @param window
	 */
	public ComponentsModel(ServiceContainerDirectoryModel sdm,IWorkbenchWindow window) {
		 serviceDirectoryModel = sdm;
		 this.window = window;
	//	setConstraint(new Rectangle(new Point(0,0),CalculateInitialContainerSize()));
	}
	
	/**
	 * @param editpart
	 */
	public void selectionChanged(Object editpart) {
		firePropertyChange(P_FOCUS, editpart, null);
	}
	
	/**
	 * @return the size the container should have without any component in it
	 */
	public static Dimension CalculateInitialContainerSize() {
		int w,h;
		ScrollPane scroll = new ScrollPane();
		h = 2*BORDER_SPACING + ((P_INIT_ROWS - 1) * SPACING) + (P_INIT_ROWS * ComponentFigure.P_HEIGHT);
		w = 2*BORDER_SPACING  + ((P_COLUMNS - 1) * SPACING) + (P_COLUMNS * ComponentFigure.P_WIDTH) + scroll.getVerticalScrollBar().getPreferredSize().width;
		return new Dimension(w,h);
	}
	/**
	 * @return the size of the view regarding the current number of components
	 */
	public Dimension CalculateViewSize() {
		int w,h;
		ScrollPane scroll = new ScrollPane();
		h = 2*BORDER_SPACING + ((P_ROWS - 1) * SPACING) + (P_ROWS  * ComponentFigure.P_HEIGHT);
		w = 2*BORDER_SPACING  + ((P_COLUMNS - 1) * SPACING) + (P_COLUMNS * ComponentFigure.P_WIDTH) + scroll.getVerticalScrollBar().getPreferredSize().width;
		return new Dimension(w,h);
	}
	
	/**
	 * update thenumber of needed rows regarding the number of components
	 */
	public void UpdateViewSize() {
		int w,h,extra_rows;
			extra_rows = getChildren().size()>(P_INIT_ROWS*P_COLUMNS)?(int) (Math.ceil((double) (getChildren().size() - (P_COLUMNS*P_INIT_ROWS))/P_COLUMNS)):0;
			P_ROWS = P_INIT_ROWS + extra_rows;
	}
	/**
	 * @return The children
	 */
	public List getChildren() {
		return children;
	}

	/**
	 * @param child
	 */
	public void removeChild(Object child) {
		children.remove(child);
		firePropertyChange(P_CHILDREN, null, null);
	}

	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	
	public IWorkbenchWindow getWindow() {
		return window;
	}
	/**
	 * @param window
	 * provide workbenc environnement to eventually display popups in the IDE
	 */
	public void setWindow(IWorkbenchWindow window) {
		this.window = window;
	}


}
