/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.util.ArrayList;
import java.util.List;

/**
* @author Maxime Carpentier
 * This model can contains container models
 *
 */
public class ContentsServiceContainerModel extends AbstractModel  {
	private EsbModel esb;
	public static final String P_CHILDREN = "_children";
	public static final String P_TOP = "_top";
	public static final String P_BOTTOM = "_bottom";
	private String position;
	private List children = new ArrayList(); 
	
	/**
	 * @param model
	 * @param scdm
	 * @param isTop
	 */
	public ContentsServiceContainerModel(EsbModel model,ServiceContainerDirectoryModel scdm,boolean isTop) {
		if (isTop)
			position = P_TOP;
		else
			position = P_BOTTOM;
		esb=model;
		this.addPropertyChangeListener(esb);
		this.addPropertyChangeListener(scdm);
	}
	
	/**
	 * @param child
	 * add containers to the model and set up the connection
	 */
	public void addChild(Object child) {
		children.add(child); 
		LineConnectionModel line1 = new LineConnectionModel();
		AbstractConnectionModel line = (AbstractConnectionModel) line1;
		
		line1.setSource((ServiceContainerModel) child);	
		line1.setTarget(esb);
		line1.attachTarget();
		line1.attachSource();
		firePropertyChange(P_CHILDREN, this, null);

	}
	
	/**
	 * @param child
	 * @param arg
	 * add containers to the model and set up the connection
	 */
	public void addChild(Object child,int arg) {
		children.add(arg,child); 
		LineConnectionModel line1 = new LineConnectionModel();
		AbstractConnectionModel line = (AbstractConnectionModel) line1;
		
		line1.setSource((ServiceContainerModel) child);	
		line1.setTarget(esb);
		line1.attachTarget();
		line1.attachSource();
		firePropertyChange(P_CHILDREN, this, null);

	}
	
	/**
	 * @param arg
	 * @param child
	 */
	public void moveChild(int arg,Object child) {
		children.remove(child);
		children.add(arg,child);
		firePropertyChange(P_CHILDREN, this, null);
	}
	/**
	 * @return the children
	 */
	public List getChildren() {
		return children;
	}

	/**
	 * @param child
	 * removes child and the associated connection
	 */
	public void removeChild(Object child) {
		List mySourceConnections = new ArrayList();
		List myTargetConnections = new ArrayList();
		mySourceConnections.addAll(((ServiceContainerModel) child).getModelSourceConnections());
		myTargetConnections.addAll(((ServiceContainerModel) child).getModelTargetConnections());
		for (int i = 0; i < mySourceConnections.size(); i++) {
			AbstractConnectionModel model = (AbstractConnectionModel) mySourceConnections.get(i);
			model.detachSource();
			model.detachTarget();
		}
		for (int i = 0; i < myTargetConnections.size(); i++) {
			 AbstractConnectionModel model =
			(AbstractConnectionModel) myTargetConnections.get(i);
			model.detachSource();
			model.detachTarget();
		}
		children.remove(child);
		firePropertyChange(P_CHILDREN, this, child);
	}

	/**
	 * @return whether the panel is the top panel or is the bottom panel
	 */
	public String getPosition() {
		return position;
	}
}