/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.tools;
 

import java.net.URL;
import java.util.ArrayList;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
 

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.petals.admin.EsbEditor;
import org.objectweb.petals.admin.actions.EsbAction;
import org.objectweb.petals.admin.states.State;
import org.w3c.dom.NamedNodeMap;

import org.w3c.dom.Node;

import org.w3c.dom.NodeList;

/**
 * @author Maxime Carpentier
 * build all available states from states.xml
 */
public class BuildStatesXML {
	private static EsbEditor esbEditor;
	
    public static ArrayList build(URL url,EsbEditor esbEditor2) throws Exception {
    	esbEditor = esbEditor2;
    	Source source = loadConfigurationFile(url);
         DOMResult domConfig = makeConfigurationAsDOM(source);
        return processBuildStates(domConfig);
    }

    private static Source loadConfigurationFile(URL url) throws Exception {
        StreamSource source = new StreamSource();
        source.setInputStream(url.openStream());
        return source;
    }

    private static DOMResult makeConfigurationAsDOM(Source source) throws Exception {
        if(source == null) return null;
        DOMResult config = null; 
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        config = new DOMResult();
        transformer.transform(source, config);
        return config;
   }

    private static ArrayList processBuildStates(DOMResult domResult) throws Exception {
    	if(domResult == null) return null;
    	ArrayList stateList = new ArrayList(); 

        // <states>
        Node rootNode = domResult.getNode().getFirstChild();
        if(! "states".equals(rootNode.getNodeName().toLowerCase()))
           throw new Exception("states element expected");

        NodeList stateElements = rootNode.getChildNodes();
        for(int j = 0; j<stateElements.getLength(); j++) {
	        	//<state name="" image="" id="">
	        	Node stateElementNode = stateElements.item(j);
	        	if(stateElementNode.getNodeType() == Node.ELEMENT_NODE)
	        		stateList.add(ProcessStateNode(stateElementNode));
        }
        return stateList;
    }
    
    private static State ProcessStateNode(Node stateElementNode) throws Exception {
        if(! "state".equals(stateElementNode.getNodeName().toLowerCase()))
            throw new Exception("state element expected");
        
        String name = getAttributeValue(stateElementNode,"name");
        if(name == null)
            throw new Exception("parameter 'name' is required for 'state' element");
        String image = getAttributeValue(stateElementNode,"image");
        if(image == null)
            throw new Exception("parameter 'image' is required for 'state' element");
        String id = getAttributeValue(stateElementNode,"id");
        if(id == null)
            throw new Exception("parameter 'id' is required for 'state' element");
        State tempState = new State(name,new Image(null, EsbEditor.class.getResourceAsStream(image)),id,esbEditor);
        
        // <actions>
        NodeList actionsNodeList = stateElementNode.getChildNodes();
    	Node actionsNode = actionsNodeList.item(0);
        for(int b = 0; (b<actionsNodeList.getLength()) && (actionsNode.getNodeType() != Node.ELEMENT_NODE); b++) {
        	actionsNode = actionsNodeList.item(b);
    }
 
    	if(actionsNode.getNodeType() != Node.ELEMENT_NODE)
    		actionsNode = stateElementNode.getChildNodes().item(2);
    		if(! "actions".equals(actionsNode.getNodeName().toLowerCase()))
           throw new Exception("actions element expected");

        
        //<<action name="" text="" method="" icon="">
        NodeList actionElements = actionsNode.getChildNodes();
        for(int a = 0; a<actionElements.getLength(); a++) {
	        	Node actionElementNode = actionElements.item(a);
	        	if(actionElementNode.getNodeType() == Node.ELEMENT_NODE)
	        		tempState.addAction(ProcessActionNode(actionElementNode));
        }
        return tempState;
    }
    
    private static EsbAction ProcessActionNode(Node actionElementNode) throws Exception {
        if(! "action".equals(actionElementNode.getNodeName().toLowerCase()))
            throw new Exception("action element expected");
        String actionName = getAttributeValue(actionElementNode,"name");
        if(actionName == null)
            throw new Exception("parameter 'name' is required for 'action' element");
        String actionText = getAttributeValue(actionElementNode,"text");
        if(actionText == null)
            throw new Exception("parameter 'text' is required for 'action' element");
        String actionMethod = getAttributeValue(actionElementNode,"method");
        if(actionMethod == null)
            throw new Exception("parameter 'method' is required for 'action' element");
        String actionIcon = getAttributeValue(actionElementNode,"icon");
        if(actionIcon == null)
            throw new Exception("parameter 'icon' is required for 'action' element");
       return new EsbAction((IWorkbenchPart) esbEditor,actionMethod,actionIcon,actionName,actionText);

    }

    

    /**

     * Return the value of the attribute in the node

     * @param n the node

     * @param attName the name of the attribute

     * @return the value of the attribute, null if not found

     */

    private static String getAttributeValue(Node n, String attName) {
        NamedNodeMap atts = n.getAttributes();
         Node att = atts.getNamedItem(attName);
        if(att != null) {
          return att.getNodeValue();
        }
        return null;
   }

    private String getTextContent(Node node) {
         NodeList list = node.getChildNodes();
         for(int i= 0; i<list.getLength();i++) {
             if(list.item(i).getNodeType() == Node.TEXT_NODE ) {
               return list.item(i).getNodeValue();
             }
         }
         return null;
    }
}


