/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.tools;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;


/**
 * @author Maxime Carpentier
 * This label provide a support for long text in a fixed size label : it cuts the text off to fit the label and adds "..."
 */
public class FixedSizeLabel extends Label {
	private int fixedSize;
	
	public FixedSizeLabel(int arg) {
		super();
		fixedSize = arg;	
	}
	
	
	protected Dimension calculateLabelSize(Dimension txtSize) {
		return new Dimension(fixedSize,super.calculateLabelSize(txtSize).height);
	}
	
	/**
	 * @param text
	 * @return
	 */
	public String getSubStringText(String text) {
		Font currentFont = new Font(null,"MS Sans Serif",8,0);
		if (currentFont == null) return null;
		int textSize = FigureUtilities.getTextWidth(text, currentFont);

		if (textSize < fixedSize) {
			return text;
		}
		int dotsWidth = FigureUtilities.getTextWidth("...", currentFont);
		int subStringLength = getLargestSubstringConfinedTo(text, currentFont,fixedSize - dotsWidth);
		return new String(text.substring(0, subStringLength) + "...");
	}
	
	
	/**
	 * @param s
	 * @param f
	 * @param availableWidth
	 * @return
	 */
	private int getLargestSubstringConfinedTo(String s, Font f, int availableWidth) {
		FontMetrics metrics = FigureUtilities.getFontMetrics(f);
		int min, max;
		float avg = metrics.getAverageCharWidth();
		min = 0;
		max = s.length() + 1;

		//The size of the current guess
		int guess = 0,
		    guessSize = 0;
		while ((max - min) > 1) {
			//Pick a new guess size
			//	New guess is the last guess plus the missing width in pixels
			//	divided by the average character size in pixels
			guess = guess + (int)((availableWidth - guessSize) / avg);

			if (guess >= max) guess = max - 1;
			if (guess <= min) guess = min + 1;

			//Measure the current guess
			guessSize = FigureUtilities.getTextExtents(s.substring(0, guess), f).width;

			if (guessSize < availableWidth)
				//We did not use the available width
				min = guess;
			else
				//We exceeded the available width
				max = guess;
		}
		return min;
	}
	
}
