/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ElementState.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component.state;


import javax.jbi.JBIException;

import org.objectweb.petals.component.LifeCycleElement;




/**
 * <code>ElementState</code> implements the <i>state</i> pattern for
 * the management of the state of the JBI elements (components and system services)
 * 
 * All implementations of this <code>ElementState</code> create
 * a thread for launching the action on the <code>LifeCycleElement</code>.
 * Methods that are not permitted in a particular <code>ElementState</code>
 * throw <code>JBIException</code> (for example, the <code>StartedState</code> throws a
 * JBIException on <i>install()</i> method)
 * @author Adrien LOUIS - EBM WebSourcing
 */
public abstract class ElementState
{
    
    public final static EmptyState empty= new EmptyState();
    public final static InstallingState installing = new InstallingState();
    public final static UninstallingState uninstalling = new UninstallingState();
    public final static InstalledState installed= new InstalledState();
    public final static StoppedState stopped= new StoppedState();
    public final static StoppingState stopping= new StoppingState();
    public final static StartedState started= new StartedState();
    public final static StartingState starting= new StartingState();
    public final static ShutDowningState shutdowning= new ShutDowningState();
    public final static UnknownState unknown= new UnknownState();
     
    protected  String name;
    
    public abstract void install(LifeCycleElement element) throws JBIException;
    public abstract void uninstall(LifeCycleElement element) throws JBIException;
    public abstract void start(LifeCycleElement element) throws JBIException;
    public abstract void stop(LifeCycleElement element) throws JBIException;
    public abstract void shutdown(LifeCycleElement element) throws JBIException;
    
    public String getName()
    {
        return name;
    }
}
