/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: InstalledState.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component.state;


import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;

import org.objectweb.petals.component.LifeCycleElement;
import org.objectweb.petals.management.activity.ActivityLauncher;
import org.objectweb.petals.management.activity.StartCommand;
import org.objectweb.petals.management.activity.UninstallCommand;
import org.objectweb.petals.util.ESBLogger;


/**
 * the Installed state.
 * Only uninstall and start are allowed.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class InstalledState extends ElementState
{

    public InstalledState()
    {
        this.name = LifeCycleMBean.SHUTDOWN;
    }
    
    /**
     * Not allowed in this state.
     * @param element the element on which the activity is executed.
     */
    public void install(LifeCycleElement element) throws JBIException
    {
        throw new JBIException("operation not permitted on Installed state");
    }

    /**
     * Uninstall the <code>LifeCycleElement</code>element
     * @param element the element on which the activity is executed.
     */
    public void uninstall(org.objectweb.petals.component.LifeCycleElement element) throws JBIException
    {
        ESBLogger.logFine(this,"uninstall("+element+")");
        
        new ActivityLauncher(element, new UninstallCommand()).start();
    }

    /**
     * Start the <code>LifeCycleElement</code>element
     * @param element the element on which the activity is executed.
     */
    public void start(org.objectweb.petals.component.LifeCycleElement element) throws JBIException
    {
        ESBLogger.logFine(this,"start("+element+")");
        
        new ActivityLauncher(element, new StartCommand()).start();
    }

    /**
     * Not allowed in this state.
     * @param element the element on which the activity is executed.
     */
    public void stop(org.objectweb.petals.component.LifeCycleElement element) throws JBIException
    {
        throw new JBIException("operation not permitted on Installed state");
    }

    /**
     * Not allowed in this state.
     * @param element the element on which the activity is executed.
     */
    public void shutdown(org.objectweb.petals.component.LifeCycleElement element) throws JBIException
    {
        throw new JBIException("operation not permitted on Installed state");
    }

}
