/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AdminService.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.management.service;

import java.util.ArrayList;

import java.util.Iterator;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.petals.component.ComponentElement;
import org.objectweb.petals.component.SystemElement;
import org.objectweb.petals.management.ManagementException;
import org.objectweb.petals.management.Manager;
import org.objectweb.petals.management.mbean.ComponentLifeCycle;
import org.objectweb.petals.management.mbean.LifeCycle;
import org.objectweb.petals.util.ESBLogger;


/**
 * TODO AdminService : check the exceptions catch
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class AdminService implements AdminServiceMBean
{
    
    protected Manager manager;
    protected MBeanServer server;
    
    public AdminService(Manager manager, MBeanServer server)
    {
        this.manager 	= manager;
        this.server 	= server;
    }
    
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents()
    {
        ESBLogger.logFine(this,"getBindingComponents()");
        
        ObjectName[] names = null;
        
        ArrayList l = new ArrayList();
        
        Map bindings = manager.getSync_bindings();
        
        synchronized(bindings)
        {
            for (Iterator iter = bindings.values().iterator(); iter.hasNext();)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                l.add(lifeCycle.getMBeanName());
            }
            names = (ObjectName[])l.toArray(new ObjectName[0]);
        }
        if (names == null)
        {
            names = new ObjectName[0];
        }
        return names;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.String)
     */
    public ObjectName getComponentByName(String name)
    {
        ESBLogger.logFine(this,"getComponentByName("+name+")");
        
        ObjectName on = null;
        
        // search in bindings
        
        LifeCycle element = (LifeCycle)manager.getSync_bindings().get(name);
        
        if(element != null)
        {
            on = element.getMBeanName();
        }
        else
        {
            // search in engines
            
            element = (LifeCycle)manager.getSync_engines().get(name);
            
            if(element != null)
            {
                on = element.getMBeanName();
            }
        }
        return on;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    public ObjectName[] getEngineComponents()
    {
        ESBLogger.logFine(this,"getEngineComponents()");
        
        ObjectName[] names = null;
        
        ArrayList l = new ArrayList();
        
        Map engines = manager.getSync_engines();
        
        synchronized(engines)
        {
            for (Iterator iter = engines.values().iterator(); iter.hasNext();)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                l.add(lifeCycle.getMBeanName());
            }
            names = (ObjectName[])l.toArray(new ObjectName[0]);
        }
        if (names == null)
        {
            names = new ObjectName[0];
        }
        return names;
    }
    
    /**
     * TODO getSystemInfo
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo()
    {
        return null;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName)
    {
        ESBLogger.logFine(this,"getSystemService("+serviceName+")");
        
        ObjectName on = null;
        
        // search in bindings
        
        LifeCycle element = (LifeCycle)manager.getSync_systemServices().get(serviceName);
        
        if(element != null)
        {
            on = element.getMBeanName();
        }
        return on;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    public ObjectName[] getSystemServices()
    {
        ESBLogger.logFine(this,"getSystemServices()");
        
        ObjectName[] names = null;
        
        ArrayList l = new ArrayList();
        
        Map services = manager.getSync_systemServices();
                
        synchronized(services)
        {
            for (Iterator iter = services.values().iterator(); iter.hasNext();)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                l.add(lifeCycle.getMBeanName());
            }
            names = (ObjectName[])l.toArray(new ObjectName[0]);
        }
        if (names == null)
        {
            names = new ObjectName[0];
        }
        return names;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName)
    {
        ESBLogger.logFine(this,"isBinding("+componentName+")");
        
        return manager.getSync_bindings().containsKey(componentName);
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName)
    {
        ESBLogger.logFine(this,"isEngine("+componentName+")");
        
        return manager.getSync_engines().containsKey(componentName);
    }
    
    ////////////////////////////////////////////////////
    // non jbi
    ////////////////////////////////////////////////////
    
    /**
     * Add a componentElement mbean as a bindingComponent mbean
     * the mbean will be exposed with the <i>mBeanName</i> attribute of the componentElement.
     * 
     * @param element the componentElement mbean to expose to JMX
     */
    public void addBindingComponent(ComponentElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"addBindingComponent("+element+")");
        
        ComponentLifeCycle clc = new ComponentLifeCycle(element);
        
        Map bindings = manager.getSync_bindings();
        
        synchronized(bindings)
        {
            try
            {
                server.registerMBean(clc, element.getMBeanName());
                
                bindings.put(element.getName(), clc);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
    
    /**
     * unexpose the BindingComponent mbean referenced by the specified name
     * @param elementName
     * @throws ManagementException
     */
    public void removeBindingComponent(ObjectName elementName) throws ManagementException
    {
        ESBLogger.logFine(this,"removeBindingComponent("+elementName+")");
        
        Map bindings = manager.getSync_bindings();
        
        synchronized(bindings)
        {
            try
            {
                server.unregisterMBean(elementName);
                
                bindings.remove(elementName);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
    
    /**
     * Add a componentElement mbean as an engineComponent mbean
     * the mbean will be exposed with the <i>mBeanName</i> attribute of the componentElement.
     * 
     * @param element the componentElement mbean to expose to JMX
     */
    public void addEngineComponent(ComponentElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"addEngineComponent("+element+")");
        
        ComponentLifeCycle clc = new ComponentLifeCycle(element);
        
        Map engines = manager.getSync_engines();
        
        synchronized(engines)
        {
            try
            {
                server.registerMBean(clc, element.getMBeanName());
                
                engines.put(element.getName(), clc);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
    
    /**
     * Unexpose the EngineComponent mbean referenced by the specified name
     * @param elementName
     * @throws ManagementException
     */
    public void removeEngineComponent(ObjectName elementName) throws ManagementException
    {
        ESBLogger.logFine(this,"removeEngineComponent("+elementName+")");
        
        Map engines = manager.getSync_engines();
                
        synchronized(engines)
        {
            try
            {
                server.unregisterMBean(elementName);
                
                engines.remove(elementName);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
    
    /**
     * Add a systemElement mbean as a systemService mbean
     * the mbean will be exposed with the <i>mBeanName</i> attribute of the systemElement.
     * 
     * @param element the systemElement mbean to expose to JMX
     */
    public void addSystemService(SystemElement element) throws ManagementException
    {
        ESBLogger.logFine(this,"addSystemService("+element+")");
        
        LifeCycle clc = new LifeCycle(element);
        
        Map systemServices = manager.getSync_systemServices();
        
        synchronized(systemServices)
        {
            try
            {
                server.registerMBean(clc, element.getMBeanName());
                
                systemServices.put(element.getName(), clc);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
    
    /**
     * Unexpose the SystemService mbean referenced by the specified name
     * @param elementName
     * @throws ManagementException
     */
    public void removeSystemService(ObjectName elementName) throws ManagementException
    {
        ESBLogger.logFine(this,"removeSystemService("+elementName+")");
        
        Map systemServices = manager.getSync_systemServices();
        
        synchronized(systemServices)
        {
            try
            {
                server.unregisterMBean(elementName);
                
                systemServices.remove(elementName);
            }
            catch (Exception e)
            {
                throw new ManagementException(e);
            }
        }
    }
}
