/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeFactoryImpl.java,v 1.3 2005/07/25 07:44:35 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.messaging;

import java.net.URI;

import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.routing.AddressFactory;
import org.objectweb.petals.util.ESBLogger;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class MessageExchangeFactoryImpl implements MessageExchangeFactory
{
    protected ComponentContextImpl context;
    
    protected DeliveryChannelImpl channel;
    
    protected AddressFactory addressFactory;
    
    protected ServiceEndpoint defaultServiceEndpoint;
    
    protected QName defaultInterfaceName;
    
    protected QName defaultServiceName;
    
    /**
     * Constructor
     * @param context
     */
    public MessageExchangeFactoryImpl(AddressFactory addressFactory, ComponentContextImpl context, DeliveryChannelImpl channel)
    {
        this.context = context;
        
        this.channel = channel;
        
        this.addressFactory = addressFactory;
    }
    
    public void setDefaultInterfaceName(QName defaultInterfaceName)
    {
        this.defaultInterfaceName = defaultInterfaceName;
    }
    
    public void setDefaultServiceEndpoint(
            ServiceEndpoint defaultServiceEndpoint)
    {
        this.defaultServiceEndpoint = defaultServiceEndpoint;
    }
    
    public void setDefaultServiceName(QName defaultServiceName)
    {
        this.defaultServiceName = defaultServiceName;
    }
    
    private String createId()
    {
        return addressFactory.createId();
    }
    
    
    
    /**
     * TODO
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(javax.xml.namespace.QName, javax.xml.namespace.QName)
     */
    public MessageExchange createExchange(QName serviceName, QName operationName)
    throws MessagingException
    {
        ESBLogger.logFine(this,"createExchange("+serviceName+","+operationName+")");
        
        channel.checkDeliveryChannelIsOpened();
        
        MessageExchangeImpl msg = new MessageExchangeImpl();
        
        msg.setService(serviceName);
        
        msg.setOperation(operationName);
        
        msg.setExchangeId(createId());
        
        return msg;
    }
    
    /**
     * TODO
     * @see javax.jbi.messaging.MessageExchangeFactory#createExchange(java.net.URI)
     */
    public MessageExchange createExchange(URI pattern)
    throws MessagingException
    {
        ESBLogger.logFine(this,"createExchange("+pattern+")");
        
        channel.checkDeliveryChannelIsOpened();
        
        MessageExchangeImpl msg = new MessageExchangeImpl();
        
        msg.setPattern(pattern);
        
        msg.setExchangeId(createId());
        
        return msg;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOnlyExchange()
     */
    public InOnly createInOnlyExchange() throws MessagingException
    {
        ESBLogger.logFine(this,"createInOnlyExchange()");
        
        channel.checkDeliveryChannelIsOpened();
        
        InOnlyImpl msg  = new InOnlyImpl();
        
        msg.setExchangeId(createId());
        
        Address sender = addressFactory.createAddressForLocalSource(context) ;
        
        msg.getItinerary().addFirst(sender);
        
        msg.getItinerary().setCurrent(0);
        
        return msg;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOptionalOutExchange()
     */
    public InOptionalOut createInOptionalOutExchange()
    throws MessagingException
    {
        ESBLogger.logFine(this,"createInOptionalOutExchange()");

        channel.checkDeliveryChannelIsOpened();
        
        InOptionalOutImpl msg  = new InOptionalOutImpl();
        
        msg.setExchangeId(createId());
        
        Address sender = addressFactory.createAddressForLocalSource(context) ;
        
        msg.getItinerary().addFirst(sender);
        
        msg.getItinerary().setCurrent(0);
        
        return msg;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchangeFactory#createInOutExchange()
     */
    public InOut createInOutExchange() throws MessagingException
    {
        ESBLogger.logFine(this,"createInOutExchange()");

        channel.checkDeliveryChannelIsOpened();
        
        InOutImpl msg  = new InOutImpl();
        
        msg.setExchangeId(createId());
        
        Address sender = addressFactory.createAddressForLocalSource(context) ;
        
        msg.getItinerary().addFirst(sender);
        
        msg.getItinerary().setCurrent(0);
        
        return msg;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchangeFactory#createRobustInOnlyExchange()
     */
    public RobustInOnly createRobustInOnlyExchange() throws MessagingException
    {
        ESBLogger.logFine(this,"createRobustInOnlyExchange()");

        channel.checkDeliveryChannelIsOpened();
        
        RobustInOnlyImpl msg  = new RobustInOnlyImpl();
        
        msg.setExchangeId(createId());
        
        Address sender = addressFactory.createAddressForLocalSource(context) ;
        
        msg.getItinerary().addFirst(sender);
        
        msg.getItinerary().setCurrent(0);
        
        return msg;
    }  
}
