/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ItineraryResolver.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;

import org.objectweb.petals.component.SystemServiceLifeCycle;
import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.messaging.MessageExchangeImpl;


/**
 * This system component is used by the container to manage
 * a message itinerary.
 * A newly initiated message only contains the recipient of 
 * the message (a <code>ServiceEndpoint</code>). 
 * 
 * Regardings defined rules, this component can alter this
 * simple itinerary by adding intermediate services,
 * such as a transformation service.
 * 
 * This component is also in charge of finding, for a given
 * message, its next destination, depending of its direction (a request or
 * a response).
 * 
 * This is the interface that have to be implemented as 
 * the container's systemService ItineraryResolver component.
 * 
 * TODO does the getNextDestination() has to be implemented or can it be codded in the container
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface ItineraryResolver extends SystemServiceLifeCycle
{
    /**
     * Manage the itinerary of this exchange.
     * Upate the itinerary with the recipient of this
     * exchange if it is newly created.
     * 
     * Insert optional services before the destination
     * if the defined rules specify it (a transformation service,...).
     * TODO manageItinerary :manage rules
     * @param msg
     * @param source
     */
    public abstract void manageItinerary(MessageExchangeImpl msg,
            ComponentContextImpl source);
    
    /**
     * Return the destination adress contained in the <code>MessageExchangeImpl</code>,
     * depending of the role of the current component (consumer or provider).
     * the next or the previous address in the <code>Itinerary</code>
     * @param exchange
     * @return
     * @throws RoutingException
     */
    public abstract Address getNextDestination(MessageExchangeImpl exchange)
    throws RoutingException;
    
    /**
     * Called by the container during initialization phase.
     * It give to this component an <code>AddressFactory</code>
     * that can be used to create address, when this component enriches
     * a message.
     * @param addressFactory
     */
    public abstract void initAddressFactory(AddressFactory addressFactory);
}