/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Router.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;


import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.transport.TransportException;
import org.objectweb.petals.transport.Transporter;
import org.objectweb.petals.util.ESBLogger;



/**
 * Routes messages to their destinations.
 * Performs an itinerary resolution, adding for example a transformatino service
 * before delivering the message to its recipient,
 * and perform an address resolution, finding the endpoint satisfying the
 * the destination information (finding an endpoint for a specified service).
 * Resolutions are based on defined rules.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class Router
{
    
    private AddressResolver addressResolver;
    
    private ItineraryResolver itineraryResolver;
    
    private Transporter transporter;
    
    /**
     * Constructs a <code>Transporter</code>.
     * @param transporter
     * @param itineraryResolver
     * @param addressResolver
     */
    public Router(Transporter transporter , ItineraryResolver itineraryResolver,AddressResolver addressResolver)
    {
        this.transporter = transporter;
        
        this.itineraryResolver = itineraryResolver;
        
        this.addressResolver = addressResolver;
    }
    
    /**
     * Sends the specified message, sent by the specified component.
     * @param exchange
     * @param source
     * @throws RoutingException
     */
    public void send(MessageExchangeImpl exchange,ComponentContextImpl source) throws RoutingException
    {
        ESBLogger.logFine(this,"send("+exchange+","+source+")");
        
        itineraryResolver.manageItinerary(exchange, source);
        
        Address destination = itineraryResolver.getNextDestination(exchange);
        
        addressResolver.resolveAddress(destination,source,exchange);
        
        try
        {
            transporter.send(exchange,destination);
        }
        catch (TransportException e)
        {
            throw new RoutingException(e);
        }
    }
    
}
