/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: StringHelper.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class StringHelper {

    /**
     * Return true if <code>String</code> is null or size is zero
     * @param s
     * @return
     */
    public static boolean isEmpty(String s)
    {
        return (s == null || s.trim().length() == 0);
    }
    
    /**
     * return true if name is not empty,
     * and respect [a-zA-Z0-9]+ regular expression
     * @param name
     * @return
     */
    public static boolean validName(String name)
    {
        if(StringHelper.isEmpty(name))
        {    
            return false;        
        }
        return name.matches("\\w+");
    }
    
    /**
     * return true if message is not null,
     * and does not contains "<" or ">"
     * @param name
     * @return
     */
    public static boolean validXMLContent(String message)
    {
        if(message == null)
        {    
            return false;        
        }
        return message.indexOf(">") == -1 && message.indexOf("<") == -1;
    }
    
    /**
     * create and return a list of String elements.
     * each element represent a classpath entry extract from
     * the classpath specified.
     * Separator in the returned String elements is appropriate to
     * the running system (File.separator)
     * 
     * <br/>The specified String is parsed by searching
     * ';' or ':' entry separator, and "/" or "\" file separator
     * @param classpath
     * @return
     */
    public static List createClassPathElements(String classpath)
    {
        if(classpath == null)
            return null;
        
        List list = new ArrayList();
        
        //check entry separator ( ':' or ';')
        boolean windowsSeparator = classpath.indexOf(';')>0;
        
        String elementSeparator = windowsSeparator?";":":";
        
        String[] elements = classpath.split(elementSeparator);
        
        //check file separator ( '\' or '/' );
        
        boolean windowsFileSeparator = classpath.indexOf('\\')>0;
        
        char fileSeparator = windowsFileSeparator?'\\':'/';
        
        char systemSeparator = File.separatorChar;
        
        for(int i = 0; i<elements.length;i++)
        {
            list.add(elements[i].replace(fileSeparator, systemSeparator));
        }
        return list;
    }
    
}
