/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ItineraryResolverImpl.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;

import java.util.HashMap;

import javax.jbi.JBIException;

import javax.jbi.messaging.MessageExchange;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.routing.AddressFactory;
import org.objectweb.petals.routing.ItineraryResolver;
import org.objectweb.petals.routing.RoutingException;
import org.objectweb.petals.util.ESBLogger;




/**
 * @author Adrien LOUIS - EBM WebSourcing
  */
public class ItineraryResolverImpl implements ItineraryResolver
{
    private AddressFactory addressFactory;
    
    
    /**
     * Manage the itinerary of this exchange.
     * Upate the itinerary with the recipient of this
     * exchange if it is newly created.
     * 
     * Insert transformation services, and so on
     * if the defined rules specify it.
     * TODO manageItinerary :manage rule.
     * @param msg
     * @param source
     */
    public void manageItinerary(MessageExchangeImpl msg,ComponentContextImpl source)
    {
        ESBLogger.logFine(this,"manageItinerary("+msg+","+source+")");
        
        // check if the itinerary is newly created :
        // the current node of the itinerary is the sender
        if(msg.getItinerary().getFirst() == msg.getItinerary().getCurrent())
        {
            // the itinerary is newly created,
            
            // insert a tranformer service
           // Address transform = new Address(null,new QName("Transformer"),null,null,null);
           // msg.getItinerary().addLast(transform);
            
            // add the recipient of the exchange as next destination
            Address destination = addressFactory.createAddressFromMessageExchange(msg);
            msg.getItinerary().addLast(destination);
        }
    }
    
    /**
     * Return the destination adress contained in the <code>MessageExchangeImpl</code>,
     * depending of the role of the current component (consumer or provider).
     * the next or the previous address in the <code>Itinerary</code>
     * @param exchange
     * @return
     * @throws RoutingException
     */
    public Address getNextDestination(MessageExchangeImpl exchange) throws RoutingException
    {
        ESBLogger.logFine(this,"getNextDestination("+exchange+")");
        
        Address destination = null;
        
        if(MessageExchange.Role.CONSUMER.equals(exchange.getRole()))
        {
            destination = exchange.getItinerary().getNext();
        }
        else if(MessageExchange.Role.PROVIDER.equals(exchange.getRole()))
        {
            destination = exchange.getItinerary().getPrevious();
        }
        else
        {
            throw new RoutingException("The role in the exchange is unknown"); 
        }  
        if(destination == null)
        {
            throw new RoutingException("The next destination address is not set in the messageExchange itinerary."); 
        }
        return destination;
    }
    
    
    /**
     * 
     */
    public void initAddressFactory(AddressFactory addressFactory)
    {
        ESBLogger.logFine(this,"initAddressFactory("+addressFactory+")");
        
        this.addressFactory = addressFactory;
    }
    
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
     */
    public void init(HashMap arg0) throws JBIException {}
    
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    public void shutDown() throws JBIException {}
    
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#start()
     */
    public void start() throws JBIException {}
    
    /**
     * do nothing
     * @see javax.jbi.component.ComponentLifeCycle#stop()
     */
    public void stop() throws JBIException {}
    
}
